# This module contains functions used by examples in Programming and Data Management for IBM SPSS Statistics 19.
# To make use of this module, copy it to your Python "site-packages" directory. This module makes use of the 
# spss, spssaux, and spssdata modules which are available once you've installed the Python Integration Plug-In.

from __future__ import with_statement

__author__ =  'SPSS Inc.'
__version__=  '3.0.0'

import spss, spssaux, spssdata, random

#pubsmarker:splitifvarexists_start
def SplitIfVarExists(varname, filespec=None):
    """Get the file, if specified, and check for the existence of 
    the specified variable. Split the dataset by the variable if it exists.
    """
    
    if filespec:
        try:
            spss.Submit("GET FILE = '%s'." %(filespec))
        except:
            raise ValueError("Cannot open file: " + filespec)

    for i in range(spss.GetVariableCount()):
       name=spss.GetVariableName(i)
       if name.lower()==varname.lower():
          spss.Submit(r"""
          SORT CASES BY %s.
          SPLIT FILE
            LAYERED BY %s.
          """ %(name,name))
          break
#pubsmarker:splitifvarexists_end

#pubsmarker:varswithformat_start
def VarsWithFormat(format):
    """Return a list of variables in the active dataset whose 
    display format has the specified string as the alphabetic part 
    of its format, e.g. "TIME".
    """
    varList=[]
    format=format.upper()
    for i in range(spss.GetVariableCount()):
        vfmt=spss.GetVariableFormat(i)
        if vfmt.rstrip("0123456789.")==format:
            varList.append(spss.GetVariableName(i))
    return varList
#pubsmarker:varswithformat_end

#pubsmarker:selectcases_start
def SelectCases(nb,crit,FPath,RPath):
    """Select a random set of cases from a data file using a
    specified criteria to determine whether a given case is
    included in the population to be sampled.
    nb is the number of cases to be selected.
    crit is the criteria to use for selecting the sample population.
    FPath is the path to the source data file.
    RPath is the path to the result file.
    """
    spss.Submit("""
    GET FILE='%(FPath)s'.
    COMPUTE casenum=$CASENUM.
    DATASET COPY temp_save.
    SELECT IF %(crit)s.
    COMPUTE draw=UNIFORM(1).
    SORT CASES BY draw.
    N OF CASES %(nb)s.
    SORT CASES BY casenum.
    MATCH FILES FILE=*
       /IN=ingrp
       /FILE=temp_save
       /BY=casenum
       /DROP=draw casenum.
    SAVE OUTFILE="%(RPath)s".
    DATASET CLOSE temp_save.
    """%locals())
#pubsmarker:selectcases_end

#pubsmarker:generatedata_start
def GenerateData(ngroups,nmembers,maxrating):
    """Generate simulated performance rating data given by each member
    of a work group to each other member of the group.
    ngroups is the number of groups.
    nmembers is the number of members in each group.
    maxrating is the maximum performance rating.
    """
    cur = spssdata.Spssdata(accessType='n')
    cur.append(spssdata.vdef("group",vfmt=["F",2,0]))
    cur.append(spssdata.vdef("rater",vfmt=["F",2,0]))
    cur.append(spssdata.vdef("ratee",vfmt=["F",2,0]))
    cur.append(spssdata.vdef("rating",vfmt=["F",2,0]))
    cur.commitdict()
    for group in range(1,ngroups+1):
        for rater in range(1,nmembers+1):
            for ratee in range(1,rater) + range(rater+1,nmembers+1):
                cur.appendvalue("group",group)
                cur.appendvalue("rater",rater)
                cur.appendvalue("ratee",ratee)
                cur.appendvalue("rating",
                                round(random.uniform(0,maxrating) + 0.5))
                cur.CommitCase()
    cur.CClose()
#pubsmarker:generatedata_end

#pubsmarker:vallabds_start
def CreateVallabDs(filespec=None, dsName=None):
    """Create a new dataset containing those variables from the active dataset
    that have value labels. The case values of the new dataset are the value
    labels from the active dataset. If the active dataset does not have value 
    labels, a message to that effect is printed in the log and no dataset 
    is created.
    filespec is the file specification of an optional file to open as the 
    active dataset.
    dsName is the optional name of the new dataset.
    """ 
    if filespec:
        try:
            spss.Submit("GET FILE = '%s'." %(filespec))
        except:
            raise ValueError("Cannot open file: " + filespec)
            
    with spss.DataStep():
       ds = spss.Dataset()
       oldname = ds.name
       newds = spss.Dataset(name=None)
       newname = newds.name
       labelsets=[]
       # Retrieve the value labels from the active dataset and create a variable
       # in the new dataset for each variable in the active one that has value 
       # labels.
       for var in ds.varlist:
          if len(var.valueLabels):
             labels = var.valueLabels.data.values()
             labelsets.append(labels)
             maxlabelwidth = max([len(item) for item in labels])
             newds.varlist.append(var.name,maxlabelwidth)
       # Populate the cases of the new dataset
       if labelsets:          
          maxnumvallabs = max([len(item) for item in labelsets])
          for i in range(maxnumvallabs):
             casevals = []
             for j in range(len(newds)):
                if i <= len(labelsets[j]) - 1:
                   vallabel = labelsets[j][i]
                   casevals.append(vallabel.ljust(newds.varlist[j].type))
                else:
                   casevals.append(None)
             newds.cases.append(casevals)
       else:
          # Discard the new dataset if no value labels were found in the active
          # dataset
          newds.close()
          print "Active dataset has no value labels."
          
    # Set the name of the new dataset to the specified name, if provided
    if labelsets and dsName:
       spss.Submit("""
       DATASET ACTIVATE %(newname)s.
       DATASET NAME %(dsName)s.
       DATASET ACTIVATE %(oldname)s.
       """ %locals())        
#pubsmarker:vallabds_end

#pubsmarker:mergeds_start
def MergeDs(ds1Name, ds2Name, keyvar):
    """Merge the case data from two datasets using a specified key variable.
    A new dataset is created with the merged cases. Only cases with matching
    values of the key variable in both datasets are retained. The order of 
    the cases in the resulting dataset is the order of the cases in the first
    specified dataset. Datasets do not need to be sorted on the key variable
    before merging.
    ds1Name and ds2Name are the names of the two datasets.
    keyvar is the name of the key variable.
    """
    with spss.DataStep():
       try:
          ds1 = spss.Dataset(name=ds1Name)
          ds2 = spss.Dataset(name=ds2Name)
          ds1keyind = ds1.varlist[keyvar].index
          ds2keyind = ds2.varlist[keyvar].index
       except:
          raise ValueError("Cannot access dataset " + ds1Name + " or dataset " + 
          ds2Name + " or one of the datasets does not contain the specified " + 
          "key variable " + keyvar)

       newds = spss.Dataset(name=None)    
       for var in ds1.varlist:
           newds.varlist.append(var.name,var.type)
       # Create a list of the index values of the variables in the second dataset,
       # excluding the key variable.
       ds2varind = [i for i in range(len(ds2.varlist)) if i != ds2keyind]
       for i in ds2varind:
           newds.varlist.append(ds2.varlist[i].name,ds2.varlist[i].type)
       # Store the case values of the key variable from the second dataset
       keys2 = [item[0] for item in ds2.cases[0:len(ds2.cases),ds2keyind]]
       # Populate the case values of the new dataset
       for row in ds1.cases:
          try:
              ds2rowindex = keys2.index(row[ds1keyind])
              newcase = row
              for i in ds2varind:
                 newcase.append(ds2.cases[ds2rowindex,i][0])
              newds.cases.append(newcase)
          except:
              pass
#pubsmarker:mergeds_end

#pubsmarker:missingsummary_start
def MissingSummary(filespec):
   """Summarize the cases with and without missing values in
   the active dataset.
   filespec is a string that identifies the file to be read.
   """
   spss.Submit("GET FILE='%s'." %(filespec))
   # Read the data and check for missing values
   data=spssdata.Spssdata()
   data.makemvchecker()
   nvalid = 0; nmissing = 0
   for row in data:
      if data.hasmissing(row):
         nmissing += 1
      else:
         nvalid +=1
   data.close()
   # Create pivot table and text block output
   spss.StartProcedure("myorganization.com.MissingSummary")
   table = spss.BasePivotTable("Case Summary","OMS table subtype")
   table.SetDefaultFormatSpec(spss.FormatSpec.Count)
   table.SimplePivotTable(rowlabels=['Valid','Missing'],
                          collabels=['Count'],
                          cells = [nvalid,nmissing])
   spss.TextBlock("Sample Text Block","A line of sample text in a text block")
   spss.EndProcedure()
#pubsmarker:missingsummary_end

#pubsmarker:groupmeans_start
def GroupMeans(groupVar,sumVar):
   """Calculate group means for a selected variable using a specified
   categorical variable to define the groups. Display the group means
   in a pivot table and add a variable for the group means to the
   active dataset.
   groupVar is the name of the categorical variable (as a string) used
   to define the groups.
   sumVar is the name of the variable (as a string) for which means are
   to be calculated.
   """
   data=spssdata.Spssdata(indexes=(groupVar,sumVar),accessType='w',
                          omitmissing=True)
   Counts={};Sums={}
   # First data pass
   for item in data:
      cat=int(item[0])
      Counts[cat]=Counts.get(cat,0) + 1
      Sums[cat]=Sums.get(cat,0) + item[1]
   for cat in sorted(Counts):
      Sums[cat]=Sums[cat]/Counts[cat]
   data.restart()
   data.append(spssdata.vdef('mean_'+sumVar+'_by_'+groupVar))
   data.commitdict()
   # Second data pass
   for item in data:
      data.casevalues([Sums[int(item[0])]])
   data.close()
   spss.StartProcedure("myorganization.com.GroupMeans")
   table = spss.BasePivotTable("Mean " + sumVar + " by " + groupVar,
                               "OMS table subtype")
   table.SimplePivotTable(rowdim=groupVar,           
                          rowlabels=[cat for cat in sorted(Counts)],
                          collabels=['mean ' + sumVar],
                          cells = [Sums[cat] for cat in Sums])

   spss.EndProcedure()
#pubsmarker:groupmeans_end
